#!/usr/bin/env python

import numpy as np
from matplotlib import animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

from logslider import LogSlider

###############################################################################

T0 = 100.
dt = 0.2

x10 = 1.
x20 = 0.

κ = 0.2
ω = 1.0

###############################################################################

π = np.pi

tt = np.linspace(-T0, T0, 2001)
t = 0.


def getxs(t):
    ωa = (ω**2 - κ**2)**0.5
    ωb = (ω**2 + κ**2)**0.5
    A = 0.5 * (x10 + x20)
    B = 0.5 * (x10 - x20)
    x1 = A * np.cos(ωa * t) + B * np.cos(ωb * t)
    x2 = A * np.cos(ωa * t) - B * np.cos(ωb * t)
    return x1, x2


plt.style.use('dark_background')

fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True, sharey=True)

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Oscylatory sprzężone")
manager.full_screen_toggle()

ax1.axhline(0, color='0.3', lw=0.7)
ax2.axhline(0, color='0.3', lw=0.7)
ax1.axvline(0, color='0.3', lw=0.7)
ax2.axvline(0, color='0.3', lw=0.7)


x1, x2 = getxs(tt)
line1, = ax1.plot(tt, x1, lw=1.5, color='#81b1d2')
line2, = ax2.plot(tt, x2, lw=1.5, color='#fa8174')

mark1, = ax1.plot([0.], [x10], 'o', color='#81b1d2', mec='0.3', lw=0.5, ms=20.)
mark2, = ax2.plot([0.], [x20], 'o', color='#fa8174', mec='0.3', lw=0.5, ms=20.)

ax2.set_xlabel('Czas $t$ [s]')
ax2.set_xlim(tt[0], tt[-1])

ax1.set_ylabel('$x_1(t)$')
ax2.set_ylabel('$x_2(t)$')

fig.tight_layout()

fig.subplots_adjust(bottom=0.18)
saxt = fig.add_axes([0.25, 0.07, 0.65, 0.03])
saxk = fig.add_axes([0.25, 0.04, 0.65, 0.03])
sax1 = fig.add_axes([0.25, 0.01, 0.25, 0.03])
sax2 = fig.add_axes([0.65, 0.01, 0.25, 0.03])

sst = lambda color='0.3': {
    'track_color': color,
    'color': color,
    'initcolor': 'black',
}

tim_slider = Slider(ax=saxt, label='Czas całkowity [s]', valmin=10., valmax=200., valinit=T0, valstep=10, **sst())
kap_slider = Slider(ax=saxk, label='Stała sprzężenia [Hz]', valmin=0., valmax=1., valstep=0.01, valinit=κ, **sst())
x10_slider = Slider(ax=sax1, label='$x_1(0)$', valmin=-1, valmax=1, valstep=1, valinit=1, **sst('#81b1d2'))
x20_slider = Slider(ax=sax2, label='$x_2(0)$', valmin=-1, valmax=1, valstep=1, valinit=0, **sst('#fa8174'))


def update():
    xx1, xx2 = getxs(tt + t)
    line1.set_data(tt, xx1)
    line2.set_data(tt, xx2)
    x1, x2 = getxs(t)
    mark1.set_ydata(x1)
    mark2.set_ydata(x2)


def on_slider(val):
    global tt, t, κ, x10, x20
    κ = kap_slider.val
    x10 = x10_slider.val
    x20 = x20_slider.val
    tt = np.arange(-tim_slider.val, tim_slider.val + 0.005, 0.01)
    ax2.set_xlim(tt[0], tt[-1])
    update()
    fig.canvas.draw_idle()


tim_slider.on_changed(on_slider)
kap_slider.on_changed(on_slider)
x10_slider.on_changed(on_slider)
x20_slider.on_changed(on_slider)


def animate(i):
    global t
    t += dt
    update()
    return line1, line2, mark1, mark2


def toggle_animation(event):
    global paused
    if event.key == ' ':
        paused = not paused
        if paused: anim.pause()
        else:
            anim.resume()
            fig.canvas.draw_idle()


anim = animation.FuncAnimation(fig, animate, interval=10, blit=True)
paused = True

fig.canvas.mpl_connect('key_press_event', toggle_animation)

plt.show()
